package example.energy.coolemall;

import schedframe.events.scheduling.EventReason;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.devices.Device;
import schedframe.resources.devices.PhysicalResource;
import schedframe.scheduling.manager.tasks.JobRegistry;
import simulator.ConfigurationOptions;
import example.energy.BaseEnergyEstimationPlugin;

public class CoolingDeviceEnergyEstimationPlugin extends BaseEnergyEstimationPlugin {

	public double estimatePowerConsumption(EnergyEvent event, JobRegistry jobRegistry,
			PhysicalResource resource) {
		
		double powerConsumption = 0;

		Device coolingDevice = (Device) resource;

		double Pchiller = 0;
		double Pdry_cooler;
		
		/*********** Pchiller ***********/
		double Cp = 1004;//constant
		
		double ncc;//DEBB
		try{
			ncc = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("coolingCoilEfficiency").get(0).getContent()).doubleValue();
		} catch (Exception e){
			ncc = 0.95;
		}
		
		double Qcooling_rated;//DEBB
		try{
			Qcooling_rated = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("coolingCapacityRated").get(0).getContent()).doubleValue();
		} catch (Exception e){
			Qcooling_rated = 3000;
		}
		
		double EERrated;//DEBB
		try{
			EERrated = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("EERRated").get(0).getContent()).doubleValue();
		} catch (Exception e){
			EERrated = 5;
		}
		
		double Tco;//DEBB
		try{
			Tco = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("CWT").get(0).getContent()).doubleValue();
		} catch (Exception e){
			Tco = 30;
		}

		double delta_Th_ex;//DEBB
		try{
			delta_Th_ex = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("deltaThEx").get(0).getContent()).doubleValue();
		} catch (Exception e){
			delta_Th_ex = 10;
		}

		double Tr_out;//from database, CFD
		try{
			Tr_out = ConfigurationOptions.outletRoomAirTempeature;
		} catch (Exception e){
			Tr_out = -1;
		}
		
		double Tr_in;//experiment
		try{
			Tr_in = ConfigurationOptions.inletRoomAirTempeature;
		} catch (Exception e){
			Tr_in = -1;
		}

		
		
		double ro = 1.168;//cosntant
		double	Vair_total;
		double mair_total;
		try{
			Vair_total = ConfigurationOptions.airFlowVolume;
		} catch (Exception e){
			Vair_total = 0;
		}
		mair_total = Vair_total * ro;
		
		
		double Qdc;
		double Qcooling;
		
		double Tev;		
		double CoolT;
		double Qcooling_nom;
		double PLR;

		double delta_T;
		double CoolPR;
		double CoolPRrated;
		double CoolPRT;
		double CoolPRPLR;
		double EER;
		
		
		Qdc = mair_total * Cp * (Tr_out - Tr_in);
		Qcooling = Qdc / ncc;

		Tev = Tr_in - delta_Th_ex;
		double Tevf = 32 + 1.8 * Tev;
		double Tcof = 32 + 1.8 * Tco;
		CoolT = 0.647177 + 0.015888 * Tevf + 0.000103 * Math.pow(Tevf, 2) - 0.004167 * Tcof + 0.000007 * Math.pow(Tcof, 2) - 0.000064 * Tevf * Tcof;
		Qcooling_nom = Qcooling_rated * CoolT;		
		PLR = Qcooling / Qcooling_nom;
		
		delta_T = Tcof - Tevf;
		CoolPRrated = 1 / EERrated;
		CoolPRT = 0.564064 - 0.011463 * Tevf + 0.000112 * Math.pow(Tevf, 2) + 0.008091 * Tcof + 0.000070 * Math.pow(Tcof, 2) - 0.000126 * Tevf * Tcof;
		CoolPRPLR = 0.023918 + 0.889469 * PLR + 0.077931 * Math.pow(PLR, 2) - 0.000264 * delta_T + 0.000007 * Math.pow(delta_T, 2) + 0.000180 * PLR * delta_T;
		CoolPR = CoolPRrated * CoolPRT * CoolPRPLR;
		EER = 1/CoolPR;
		
		Pchiller = Qcooling/EER;

			
		

		/*********** Pdry_cooler ***********/
		double delta_Th_dry_cooler;//DEBB
		try{
			delta_Th_dry_cooler = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("deltaThDryCooler").get(0).getContent()).doubleValue();
		} catch (Exception e){
			delta_Th_dry_cooler = 10;
		}
		
		double Tamb;//experiment
		try{
			Tamb = ConfigurationOptions.ambientAirTempeature;
		} catch (Exception e){
			Tamb = -1;
		}
		
		Pdry_cooler = 0.02 * Qcooling;
		
		
		//if data not present in DB than don't take Pchiller into account
		if(Vair_total != -1 && Tr_in != -1 && Tr_out != -1) {
			if(Tamb != -1 && Tev - Tamb > 0 && delta_Th_dry_cooler > Tev - Tamb)
				powerConsumption = Pdry_cooler;
			else
				powerConsumption = Pchiller;	
		}
		
		

		
		return powerConsumption;
	}
	
	public double estimateAirThroughput(EnergyEvent event, JobRegistry jobRegistry, PhysicalResource resource) {
		double airThroughput = 0;
		try {
			if(event.getReason() == EventReason.SIM_INIT)
				return airThroughput;
				//airThroughput = resource.getAirThroughputInterface().getAirFlow(StandardAirThroughputStateName.FAN_OFF);
			else
				airThroughput = resource.getAirThroughputInterface().getAirFlow(resource.getAirThroughputInterface().getAirThroughputState());
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return airThroughput;
	}

}
